import type { Temperament as TemperamentData } from "./schema.js";
export type { TemperamentData };
/** The size of an octave in cents. */
export declare const OCTAVE_SIZE = 1200;
/**
 * Contains a complete description of a musical temperament.
 *
 * Most interaction with a `Temperament` should be through its public methods,
 * documented below.  However, "metadata" properties (such as the name of the
 * temperament) are available for direct access, since changing them has no
 * effect on the internal structure of the temperament.  These metadata
 * properties correspond directly to properties in the input data, and are
 * documented below.  Please note that a metadata property may be `undefined`
 * if it was not defined in the input data.
 */
export declare class Temperament {
    /** The name of the temperament. */
    readonly name: string;
    /** A short description of the temperament. */
    readonly description?: string;
    /** The source of the temperament data (e.g. a URL). */
    readonly source?: string;
    private readonly _octaveBaseName;
    private readonly _referenceName;
    private readonly _referenceOctave;
    private _referencePitch;
    private readonly _noteNames;
    private readonly _offsets;
    /**
     * Creates a new temperament.
     *
     * @param data - the temperament data, in the format defined by the schema
     */
    constructor(data: TemperamentData);
    /**
     * Returns an array of the note names defined in the temperament.
     *
     * @returns the note names, sorted in increasing order of pitch starting with
     * the octave base
     */
    get noteNames(): string[];
    /**
     * Returns the name of the octave base note.
     *
     * @returns the name of the octave base note
     */
    get octaveBaseName(): string;
    /**
     * Returns the name of the reference note.
     *
     * @returns the name of the reference note
     */
    get referenceName(): string;
    /**
     * Returns the octave number of the reference note.
     *
     * @returns the octave number of the reference note
     */
    get referenceOctave(): number;
    /**
     * Returns the reference pitch.
     *
     * @returns the reference pitch, in Hz
     */
    get referencePitch(): number;
    /**
     * Sets the reference pitch.
     *
     * @param pitch - the reference pitch (in Hz)
     * @throws {@link Error} if `pitch` is not positive
     */
    set referencePitch(pitch: number);
    /**
     * Returns the closest note to the given pitch (in Hz), along with the pitch
     * difference (in cents).
     *
     * @param pitch - the pitch of the note to identify (in Hz)
     * @returns a tuple containing the note name as its first element and the
     * offset (in cents) from that note as its second element
     * @throws {@link Error} if `pitch` is not positive
     */
    getNoteNameFromPitch(pitch: number): [string, number];
    /**
     * Return an array with octave numbers in order, forming a range with the
     * given radius around the reference octave.
     *
     * @param radius - the number of octaves on either end of the reference octave
     * to include. Must be non-negative.
     * @returns a range of octave numbers
     * @throws {@link Error} if `radius` is negative
     */
    getOctaveRange(radius: number): number[];
    /**
     * Returns the offset of the given note.
     *
     * @param note - the name of the note
     * @param octave - the octave number of the note
     * @returns the offset (in cents), relative to the reference pitch
     * @throws {@link Error} if `note` is not defined
     */
    getOffset(note: string, octave: number): number;
    /**
     * Returns the pitch of the given note.
     *
     * @param note - the name of the note
     * @param octave - the octave number of the note
     * @returns the pitch (in Hz)
     * @throws {@link Error} if `note` is not defined
     */
    getPitch(note: string, octave: number): number;
    /**
     * Returns temperament data in the format defined by the schema.
     *
     * The returned data is not necessarily equal to the data object passed in the
     * constructor, but it is equivalent as a temperament. For example, the way
     * the note offsets are expressed may be different.
     *
     * @returns temperament data describing this temperament, suitable for
     * conversion to JSON
     */
    toJSON(): TemperamentData;
}
