var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import Ajv from "ajv";
const title = "Temperament";
const description = "A complete description of a musical temperament with additional metadata.";
const type = "object";
const required = [
  "name",
  "octaveBaseName",
  "referencePitch",
  "referenceName",
  "referenceOctave",
  "notes"
];
const properties = {
  name: {
    description: "The name of the temperament.",
    type: "string"
  },
  description: {
    description: "A description of the temperament.",
    type: "string"
  },
  source: {
    description: "The source from which the temperament data was obtained.",
    type: "string"
  },
  octaveBaseName: {
    description: "The name of the base note of the octave.",
    type: "string"
  },
  referencePitch: {
    description: "The frequency (in Hz) of the reference pitch.",
    type: "number",
    exclusiveMinimum: 0
  },
  referenceName: {
    description: "The note name of the reference pitch.",
    type: "string"
  },
  referenceOctave: {
    description: "The octave number of the reference pitch.",
    type: "integer"
  },
  notes: {
    description: "A mapping of note names to a description of each note.",
    type: "object",
    minProperties: 1,
    additionalProperties: {
      description: "A description of the note, as an offset from another note.",
      type: "array",
      minItems: 2,
      maxItems: 2,
      items: [
        {
          description: "The base note from which to offset.",
          type: "string"
        },
        {
          description: "The offset (in cents) from the base note.",
          type: "number"
        }
      ]
    }
  }
};
var schema = {
  title,
  description,
  type,
  required,
  properties
};
const ajv = new Ajv();
const validate = ajv.compile(schema);
const OCTAVE_SIZE = 1200;
function defineOffset(offsets, note, offset) {
  const existingOffset = offsets.get(note);
  if (existingOffset !== void 0 && (existingOffset - offset) % OCTAVE_SIZE !== 0) {
    throw new Error(`Conflicting definition for '${note}' found`);
  }
  offsets.set(note, offset);
}
function computeOffsets(temperament) {
  const notes = temperament.notes;
  const offsets = new Map([[temperament.referenceName, 0]]);
  const todo = [[temperament.referenceName, 0]];
  while (todo.length !== 0) {
    const [currentName, currentOffset] = todo[0];
    if (Object.prototype.hasOwnProperty.call(notes, currentName)) {
      const [name, offset] = notes[currentName];
      const computedOffset = currentOffset - offset;
      if (name !== currentName && !offsets.has(name)) {
        todo.push([name, computedOffset]);
      }
      defineOffset(offsets, name, computedOffset);
    }
    Object.keys(notes).filter((name) => notes[name][0] === currentName).forEach((name) => {
      const offset = notes[name][1];
      const computedOffset = currentOffset + offset;
      if (name !== currentName && !offsets.has(name)) {
        todo.push([name, computedOffset]);
      }
      defineOffset(offsets, name, computedOffset);
    });
    todo.shift();
  }
  Object.keys(notes).forEach((name) => {
    if (!offsets.has(name)) {
      throw new Error(`Not able to determine the pitch of '${name}'`);
    }
  });
  const originalOctaveBaseOffset = offsets.get(temperament.octaveBaseName);
  if (originalOctaveBaseOffset === void 0) {
    throw new Error("Octave base not defined as a note");
  }
  const octaveBaseOffset = originalOctaveBaseOffset > 0 ? originalOctaveBaseOffset % OCTAVE_SIZE - OCTAVE_SIZE : originalOctaveBaseOffset % OCTAVE_SIZE;
  offsets.set(temperament.octaveBaseName, octaveBaseOffset);
  offsets.forEach((offset, name) => {
    let relative = offset - octaveBaseOffset;
    relative %= OCTAVE_SIZE;
    if (relative < 0) {
      relative += OCTAVE_SIZE;
    }
    offsets.set(name, octaveBaseOffset + relative);
  });
  return offsets;
}
class Temperament {
  constructor(data) {
    __publicField(this, "name");
    __publicField(this, "description");
    __publicField(this, "source");
    __publicField(this, "_octaveBaseName");
    __publicField(this, "_referenceName");
    __publicField(this, "_referenceOctave");
    __publicField(this, "_referencePitch");
    __publicField(this, "_noteNames");
    __publicField(this, "_offsets");
    if (!validate(data)) {
      throw new TypeError(`Incorrect temperament format: ${ajv.errorsText(validate.errors)}`);
    }
    this.name = data.name;
    this.description = data.description;
    this.source = data.source;
    this._octaveBaseName = data.octaveBaseName;
    this._referenceName = data.referenceName;
    this._referenceOctave = data.referenceOctave;
    this._referencePitch = data.referencePitch;
    this._offsets = computeOffsets(data);
    this._noteNames = [...this._offsets.entries()].sort((a, b) => a[1] - b[1]).map((entry) => entry[0]);
  }
  get noteNames() {
    return [...this._noteNames];
  }
  get octaveBaseName() {
    return this._octaveBaseName;
  }
  get referenceName() {
    return this._referenceName;
  }
  get referenceOctave() {
    return this._referenceOctave;
  }
  get referencePitch() {
    return this._referencePitch;
  }
  set referencePitch(pitch) {
    if (pitch <= 0)
      throw new Error("Pitch must be positive");
    this._referencePitch = pitch;
  }
  getNoteNameFromPitch(pitch) {
    if (pitch <= 0)
      throw new Error("Pitch must be positive");
    const baseOffset = this.getOffset(this._octaveBaseName, this._referenceOctave);
    let offset = Math.log2(pitch / this._referencePitch) * OCTAVE_SIZE;
    offset = (offset - baseOffset) % OCTAVE_SIZE + baseOffset;
    if (offset < baseOffset) {
      offset += OCTAVE_SIZE;
    }
    let start = 0;
    let end = this._noteNames.length;
    while (end - start > 1) {
      const mid = Math.trunc((end + start) / 2);
      const midOffset = this.getOffset(this._noteNames[mid], this._referenceOctave);
      if (offset > midOffset) {
        start = mid;
      } else if (offset < midOffset) {
        end = mid;
      } else {
        return [this._noteNames[mid], 0];
      }
    }
    const startNote = this._noteNames[start];
    const startDifference = offset - this.getOffset(startNote, this._referenceOctave);
    let endNote, endDifference;
    if (end === this._noteNames.length) {
      endNote = this._noteNames[0];
      endDifference = offset - this.getOffset(endNote, this._referenceOctave) - OCTAVE_SIZE;
    } else {
      endNote = this._noteNames[end];
      endDifference = offset - this.getOffset(endNote, this._referenceOctave);
    }
    if (Math.abs(startDifference) < Math.abs(endDifference)) {
      return [startNote, startDifference];
    } else {
      return [endNote, endDifference];
    }
  }
  getOctaveRange(radius) {
    if (radius < 0)
      throw new Error("Radius must not be negative");
    const start = this._referenceOctave - radius;
    const end = this._referenceOctave + radius;
    const octaves = [];
    for (let i = start; i <= end; i++) {
      octaves.push(i);
    }
    return octaves;
  }
  getOffset(note, octave) {
    const offset = this._offsets.get(note);
    if (offset === void 0) {
      throw new Error(`Note '${note}' is not defined`);
    }
    return offset + (octave - this._referenceOctave) * OCTAVE_SIZE;
  }
  getPitch(note, octave) {
    return this._referencePitch * 2 ** (this.getOffset(note, octave) / OCTAVE_SIZE);
  }
  toJSON() {
    return {
      name: this.name,
      description: this.description,
      source: this.source,
      octaveBaseName: this.octaveBaseName,
      referencePitch: this.referencePitch,
      referenceName: this.referenceName,
      referenceOctave: this.referenceOctave,
      notes: Object.fromEntries([...this._offsets.entries()].map(([note, offset]) => [
        note,
        [this.referenceName, offset]
      ]))
    };
  }
}
export { OCTAVE_SIZE, Temperament };
//# sourceMappingURL=temperament.es.mjs.map
